@@ -18,13 +18,14 @@ function(
 	sgp.config=NULL,
 	sgp.percentiles.equated=FALSE,
 	SGPt=NULL,
+	fix.duplicates=NULL,
 	parallel.config=NULL) {
 
 	started.at <- proc.time()
 	messageSGP(paste("Started combineSGP", prettyDate()))
 
-	ID <- CONTENT_AREA <- YEAR <- GRADE <- YEAR_INTEGER_TMP <- ACHIEVEMENT_LEVEL <- CATCH_UP_KEEP_UP_STATUS_INITIAL <- MOVE_UP_STAY_UP_STATUS_INITIAL <- VALID_CASE <- NULL
-	MOVE_UP_STAY_UP_STATUS <- CATCH_UP_KEEP_UP_STATUS <- ACHIEVEMENT_LEVEL_PRIOR <- target.type <- SGP_PROJECTION_GROUP <- NULL
+	ID <- CONTENT_AREA <- YEAR <- GRADE <- YEAR_INTEGER_TMP <- ACHIEVEMENT_LEVEL <- CATCH_UP_KEEP_UP_STATUS_INITIAL <- MOVE_UP_STAY_UP_STATUS_INITIAL <- VALID_CASE <- N <- NULL
+	MOVE_UP_STAY_UP_STATUS <- CATCH_UP_KEEP_UP_STATUS <- ACHIEVEMENT_LEVEL_PRIOR <- target.type <- SGP_PROJECTION_GROUP <- DUPS_FLAG <- i.DUPS_FLAG <- SCALE_SCORE <- SGP_NORM_GROUP_SCALE_SCORES <- NULL
 
 	tmp.messages <- NULL
 
@@ -51,6 +52,9 @@ function(
 		}
 	}
 
+
+	### SGP_Configuration arguments
+
 	### Create SGP_TARGET_CONTENT_AREA in certain cases
 
 	if (is.null(sgp.target.content_areas) & any(sapply(SGP::SGPstateData[[state]][["SGP_Configuration"]][["content_area.projection.sequence"]], function(x) uniqueN(x)) > 1)) {
@@ -91,6 +95,12 @@ function(
 		if (sgp.target.scale.scores) sgp.projections.equated <- NULL
 	}
 
+	### fix.duplicates
+
+	if (is.null(fix.duplicates) & !is.null(SGP::SGPstateData[[state]][["SGP_Configuration"]][["fix.duplicates"]])) {
+		fix.duplicates <- SGP::SGPstateData[[state]][["SGP_Configuration"]][["fix.duplicates"]]
+	}
+
 
 	### Utility functions
 
@@ -167,7 +177,9 @@ function(
 	catch_keep_move_functions <- c(min, max)
 
 	getTargetData <- function(tmp.target.data, projection_group.iter, tmp.target.level.names) {
-		if ("YEAR_WITHIN" %in% names(tmp.target.data)) tmp.var.names <- c("ID", "CONTENT_AREA", "YEAR", "YEAR_WITHIN") else tmp.var.names <- c("ID", "CONTENT_AREA", "YEAR")
+		if ("YEAR_WITHIN" %in% names(tmp.target.data)) {
+			tmp.var.names <- c("ID", "CONTENT_AREA", "YEAR", "YEAR_WITHIN", intersect(names(tmp.target.data), c("GRADE", "SGP_PROJECTION_GROUP_SCALE_SCORES")))
+		} else tmp.var.names <- c("ID", "CONTENT_AREA", "YEAR", intersect(names(tmp.target.data), c("GRADE", "SGP_PROJECTION_GROUP_SCALE_SCORES")))
 		tmp.data <- tmp.target.data[SGP_PROJECTION_GROUP==projection_group.iter, c(tmp.var.names, tmp.target.level.names), with=FALSE]
 		na.omit(tmp.data, cols=grep("MOVE_UP_STAY_UP", tmp.target.level.names, invert=TRUE, value=TRUE))
 	}
@@ -228,9 +240,32 @@ function(
 			tmp.data <- getPreferredSGP(tmp.data, state)
 		}
 
-		variables.to.merge <- setdiff(names(tmp.data),  key(slot.data))
-		tmp.index <- slot.data[tmp.data[,key(slot.data), with=FALSE], which=TRUE]
-		slot.data[tmp.index, (variables.to.merge):=tmp.data[, variables.to.merge, with=FALSE]]
+		if (!is.null(fix.duplicates) & any(grepl("_DUPS_[0-9]*", tmp.data[["ID"]]))) {
+			##  Strip ID of the _DUPS_ Flag, but keep in a seperate variable (used to merge subsequently)
+			invisible(tmp.data[, DUPS_FLAG := gsub(".*_DUPS_", "", ID)])
+			invisible(tmp.data[!grepl("_DUPS_[0-9]*", ID), DUPS_FLAG := NA])
+			invisible(tmp.data[, ID := gsub("_DUPS_[0-9]*", "", ID)])
+
+			##  Extend the slot.data if any new rows are required (e.g. dups in prior years) - if not still merge in DUPS_FLAG.
+			slot.data.extension <- tmp.data[!is.na(DUPS_FLAG), c(key(slot.data), "SGP_NORM_GROUP_SCALE_SCORES", "DUPS_FLAG"), with=FALSE]
+			tmp.split <- strsplit(as.character(slot.data.extension[["SGP_NORM_GROUP_SCALE_SCORES"]]), "; ")
+			invisible(slot.data.extension[, SCALE_SCORE := as.numeric(sapply(tmp.split, function(x) rev(x)[1]))])
+			invisible(slot.data.extension[, SGP_NORM_GROUP_SCALE_SCORES := NULL])
+			if ("DUPS_FLAG" %in% names(slot.data)) flag.fix <- TRUE else flag.fix <- FALSE
+			slot.data <- slot.data.extension[slot.data, on=c(key(slot.data),"SCALE_SCORE"), allow.cartesian=TRUE]
+			if (flag.fix) { # Merge together DUPS_FLAG from previous years
+				invisible(slot.data[!is.na(i.DUPS_FLAG) & is.na(DUPS_FLAG), DUPS_FLAG := i.DUPS_FLAG])
+				invisible(slot.data[, i.DUPS_FLAG := NULL])
+			}
+
+			##  Get the row index for variable merge.
+			tmp.index <- slot.data[tmp.data[, c(getKey(slot.data), "GRADE", "DUPS_FLAG"), with=FALSE], which=TRUE, on=c(getKey(slot.data), "GRADE", "DUPS_FLAG")]
+		} else {
+			tmp.index <- slot.data[tmp.data[, key(tmp.data), with=FALSE], which=TRUE, on=key(tmp.data)]
+		}
+
+		variables.to.merge <- setdiff(names(tmp.data), c(getKey(slot.data), "GRADE"))
+		invisible(slot.data[tmp.index, (variables.to.merge):=tmp.data[, variables.to.merge, with=FALSE]])
 
 		setkeyv(slot.data, getKey(slot.data))
 	}
@@ -271,9 +306,32 @@ function(
 			tmp.data <- getPreferredSGP(tmp.data, state, type="BASELINE")
 		}
 
-		variables.to.merge <- setdiff(names(tmp.data),  key(slot.data))
-		tmp.index <- slot.data[tmp.data[,key(slot.data), with=FALSE], which=TRUE]
-		slot.data[tmp.index, (variables.to.merge):=tmp.data[, variables.to.merge, with=FALSE]]
+		if (!is.null(fix.duplicates) & any(grepl("_DUPS_[0-9]*", tmp.data[["ID"]]))) {
+			##  Strip ID of the _DUPS_ Flag, but keep in a seperate variable (used to merge subsequently)
+			invisible(tmp.data[, DUPS_FLAG := gsub(".*_DUPS_", "", ID)])
+			invisible(tmp.data[!grepl("_DUPS_[0-9]*", ID), DUPS_FLAG := NA])
+			invisible(tmp.data[, ID := gsub("_DUPS_[0-9]*", "", ID)])
+
+			##  Extend the slot.data if any new rows are required (e.g. dups in prior years) - if not still merge in DUPS_FLAG.
+			slot.data.extension <- tmp.data[!is.na(DUPS_FLAG), c(key(slot.data), "SGP_NORM_GROUP_SCALE_SCORES", "DUPS_FLAG"), with=FALSE]
+			tmp.split <- strsplit(as.character(slot.data.extension[["SGP_NORM_GROUP_SCALE_SCORES"]]), "; ")
+			invisible(slot.data.extension[, SCALE_SCORE := as.numeric(sapply(tmp.split, function(x) rev(x)[1]))])
+			invisible(slot.data.extension[, SGP_NORM_GROUP_SCALE_SCORES := NULL])
+			if ("DUPS_FLAG" %in% names(slot.data)) flag.fix <- TRUE else flag.fix <- FALSE
+			slot.data <- slot.data.extension[slot.data, on=c(key(slot.data),"SCALE_SCORE"), allow.cartesian=TRUE]
+			if (flag.fix) { # Merge together DUPS_FLAG from previous years
+				invisible(slot.data[!is.na(i.DUPS_FLAG) & is.na(DUPS_FLAG), DUPS_FLAG := i.DUPS_FLAG])
+				invisible(slot.data[, i.DUPS_FLAG := NULL])
+			}
+
+			##  Get the row index for variable merge.
+			tmp.index <- slot.data[tmp.data[, c(getKey(slot.data), "GRADE", "DUPS_FLAG"), with=FALSE], which=TRUE, on=c(getKey(slot.data), "GRADE", "DUPS_FLAG")] #
+		} else {
+			tmp.index <- slot.data[tmp.data[, key(tmp.data), with=FALSE], which=TRUE, on=key(tmp.data)]
+		}
+
+		variables.to.merge <- setdiff(names(tmp.data), c(getKey(slot.data), "GRADE"))
+		invisible(slot.data[tmp.index, (variables.to.merge):=tmp.data[, variables.to.merge, with=FALSE]])
 
 		setkeyv(slot.data, getKey(slot.data))
 	}
@@ -311,14 +369,63 @@ function(
 
 		for (target.type.iter in target.args[['target.type']]) {
 			for (target.level.iter in target.args[['target.level']]) {
-				tmp.data <- getTargetSGP(sgp_object, content_areas, state, years, target.type.iter, target.level.iter, max.sgp.target.years.forward)
-				if (any(duplicated(tmp.data, by=key(tmp.data)))) {
+
+				if (!is.null(fix.duplicates)) {
+					if (!sgp.percentiles & !sgp.percentiles.baseline) messageSGP("The fix.duplicates='KEEP.ALL' functionality requires that sgp.percentiles = TRUE or percentiles results have already been merged into @Data.")
+					##  Seperate out prior score history of slot.data
+					tmp.split <- strsplit(as.character(slot.data[["SGP_NORM_GROUP_SCALE_SCORES"]]), "; ")
+					num.scores <- max(sapply(seq_along(tmp.split), function(f) length(tmp.split[[f]])))
+					if (num.scores > 2) {
+						for (tmp.prior in tail(seq(num.scores), -2)) {
+							invisible(slot.data[, paste0("SCALE_SCORE_PRIOR_", tmp.prior-1L) := as.numeric(sapply(tmp.split, function(x) rev(x)[tmp.prior]))])
+				}}}
+
+				tmp.data <- getTargetSGP(sgp_object, slot.data, content_areas, state, years, target.type.iter, target.level.iter, max.sgp.target.years.forward, fix.duplicates=fix.duplicates)
+
+				if (!is.null(fix.duplicates)) dup.by <- c(key(tmp.data), grep("SCALE_SCORE$|SCALE_SCORE_PRIOR", names(tmp.data), value=TRUE)) else dup.by <- key(tmp.data)
+
+				if (any(duplicated(tmp.data, by=dup.by))) {
 					duplicated.projections.tf <- TRUE
 					tmp.data <- getPreferredSGP(tmp.data, state, type="TARGET")
 				} else duplicated.projections.tf <- FALSE
-				variables.to.merge <- setdiff(names(tmp.data),  key(slot.data))
-				tmp.index <- slot.data[tmp.data[,intersect(key(slot.data), names(tmp.data)), with=FALSE], which=TRUE]
-				slot.data[tmp.index, (variables.to.merge):=tmp.data[, variables.to.merge, with=FALSE]]
+
+				if (!is.null(fix.duplicates) & any(grepl("_DUPS_[0-9]*", tmp.data[["ID"]]))) {
+					##  Strip ID of the _DUPS_ Flag, Don't use this as DUPS_FLAG (merge in later from SGPercentiles)
+					invisible(tmp.data[, ID := gsub("_DUPS_[0-9]*", "", ID)])
+
+					##  Get the row index for variable merge.
+					if (grepl('lagged', target.type.iter)) {
+						tmp.index <- slot.data[
+							tmp.data[, c(intersect(getKey(slot.data), names(tmp.data)), "DUPS_FLAG", grep("SCALE_SCORE_PRIOR", names(tmp.data), value=TRUE)), with=FALSE, nomatch=NA],
+							which=TRUE, on=c(getKey(slot.data), "DUPS_FLAG", grep("SCALE_SCORE_PRIOR", names(tmp.data), value=TRUE))]
+
+						no_match <- tmp.data[which(is.na(tmp.index)),] # usually current year score is NA - still get a lagged projection, but no SGP (& therefore no prior score to merge on)
+						if (nrow(no_match) > 0) {
+							no_match.index <- slot.data[no_match[, intersect(getKey(slot.data), names(no_match)), with=FALSE, nomatch=NA], which=TRUE, on=getKey(slot.data)]
+							if (length(no_match.index) == length(tmp.index[which(is.na(tmp.index))])) {
+								tmp.index[which(is.na(tmp.index))] <- no_match.index
+							} else stop("Error in matching LAGGED projections with duplicates in data (most likely student records with a current year SCALE_SCORE == NA).")
+						}
+					} else {
+						setnames(tmp.data, "SGP_PROJECTION_GROUP_SCALE_SCORES", "SGP_PROJECTION_GROUP_SCALE_SCORES_CURRENT")
+
+						tmp.index <- slot.data[
+							tmp.data[, c(intersect(getKey(slot.data), names(tmp.data)), "DUPS_FLAG", grep("SCALE_SCORE$|SCALE_SCORE_PRIOR", names(tmp.data), value=TRUE)), with=FALSE, nomatch=NA],
+							which=TRUE, on=c(getKey(slot.data), "DUPS_FLAG", grep("SCALE_SCORE$|SCALE_SCORE_PRIOR", names(tmp.data), value=TRUE))]
+
+						no_match <- tmp.data[which(is.na(tmp.index)),] # usually current year score is NA - still get a lagged projection, but no SGP (& therefore no prior score to merge on)
+						if (nrow(no_match) > 0) {
+							no_match.index <- slot.data[no_match[, intersect(getKey(slot.data), names(no_match)), with=FALSE, nomatch=NA], which=TRUE, on=intersect(getKey(slot.data), names(no_match))]
+							if (length(no_match.index) == length(tmp.index[which(is.na(tmp.index))])) {
+								tmp.index[which(is.na(tmp.index))] <- no_match.index
+							} else stop("Error in matching STRAIGHT (CURRENT) projections with duplicates in data (most likely student records with a current year SCALE_SCORE == NA).")
+						}
+					}
+				} else {
+					tmp.index <- slot.data[tmp.data[,intersect(getKey(slot.data), names(tmp.data)), with=FALSE], which=TRUE, on=dup.by]
+				}
+				variables.to.merge <- setdiff(names(tmp.data), c(getKey(slot.data), "DUPS_FLAG", grep("SCALE_SCORE$|SCALE_SCORE_PRIOR", names(tmp.data), value=TRUE)))
+				invisible(slot.data[tmp.index, (variables.to.merge):=tmp.data[, variables.to.merge, with=FALSE]])
 			}
 		}
 		if (duplicated.projections.tf) {
@@ -335,8 +442,8 @@ function(
 
 		if (!is.null(sgp.target.content_areas) && sgp.target.content_areas) {
 			for (my.sgp.target.content_area.iter in target.args[['my.sgp.target.content_area']]) {
-				slot.data[!is.na(get(target.args[['my.sgp.target']][1])), target.args[['my.sgp.target.content_area']]:=
-					getTargetSGPContentArea(GRADE[1], CONTENT_AREA[1], state, max.sgp.target.years.forward, target.args[['my.sgp.target.content_area.iter']]),
+				slot.data[!is.na(get(target.args[['my.sgp.target']][1])), target.args[['my.sgp.target.content_area']] :=
+					getTargetSGPContentArea(GRADE[1], CONTENT_AREA[1], state, max.sgp.target.years.forward, my.sgp.target.content_area.iter),
 					by=list(GRADE, CONTENT_AREA)]
 			}
 		}
@@ -438,14 +545,21 @@ function(
 		for (target.type.iter in target.args[['sgp.target.scale.scores.types']]) {
 			for (target.level.iter in target.args[['target.level']]) {
 				tmp.target.list[[paste(target.type.iter, target.level.iter)]] <-
-					data.table(getTargetSGP(sgp_object, content_areas, state, years, target.type.iter, target.level.iter, max.sgp.target.years.forward, return.lagged.status=FALSE),
+					data.table(getTargetSGP(sgp_object, slot.data, content_areas, state, years, target.type.iter, target.level.iter, max.sgp.target.years.forward, return.lagged.status=FALSE, fix.duplicates=fix.duplicates),
 						key=c(getKey(sgp_object), "SGP_PROJECTION_GROUP"))
 			}
 		}
-		tmp.target.data <- data.table(Reduce(function(x, y) merge(x, y, all=TRUE), tmp.target.list[!sapply(tmp.target.list, function(x) dim(x)[1]==0)],
-			accumulate=FALSE), key=c("VALID_CASE", "CONTENT_AREA", "YEAR", "ID"))
+		tmp.target.data <- data.table(Reduce(function(x, y) merge(x, y, all=TRUE, by=intersect(names(y), names(x))), tmp.target.list[!sapply(tmp.target.list, function(x) dim(x)[1]==0)],
+			accumulate=FALSE), key=getKey(slot.data))
 
-		for (projection_group.iter in unique(tmp.target.data, by='SGP_PROJECTION_GROUP')[['SGP_PROJECTION_GROUP']]) {
+		if (!is.null(fix.duplicates)) {
+			if (any(grepl("_DUPS_[0-9]*", tmp.target.data[["ID"]]))) {
+				invisible(tmp.target.data[, ID := gsub("_DUPS_[0-9]*", "", ID)])
+				invisible(tmp.target.data[!is.na(DUPS_FLAG), N := seq.int(.N), by=c(getKey(tmp.target.data))])
+			}
+		}
+
+		for (projection_group.iter in unique(tmp.target.data[['SGP_PROJECTION_GROUP']])) {
 			for (target.type.iter in target.args[['sgp.target.scale.scores.types']]) {
 				tmp.target.level.names <-
 					as.character(sapply(target.args[['target.level']], function(x) getTargetName(state, target.type.iter, x, max.sgp.target.years.forward, "SGP_TARGET", projection.unit.label, projection_group.iter)))
@@ -464,13 +578,16 @@ function(
 					projection_group.identifier=projection_group.iter,
 					sgp.projections.equated=if (grepl("baseline", target.type.iter)) NULL else sgp.projections.equated,
 					SGPt=SGPt,
+					fix.duplicates=fix.duplicates,
 					parallel.config=parallel.config)
 			}
 		}
 	} ### END if (sgp.target.scale.scores)
 
+	### Final clean and put slot.data into @Data slot
 
-	### Put slot.data into @Data slot
+	if ("DUPS_FLAG" %in% names(slot.data)) invisible(slot.data[, DUPS_FLAG := NULL])
+	if (any(grepl("SCALE_SCORE_PRIOR_[0-9]", names(slot.data)))) invisible(slot.data[, grep("SCALE_SCORE_PRIOR_[0-9]", names(slot.data), value=TRUE) := NULL])
 
 	setkeyv(slot.data, getKey(slot.data))
 	sgp_object@Data <- slot.data
