@@ -1,6 +1,6 @@
 `yearIncrement` <-
-function(year, 
-	increment, 
+function(year,
+	increment,
 	lag=0) {
 
 	if (is.null(year)) return(NULL)
@@ -8,9 +8,9 @@ function(year,
 	if (identical(year, "BASELINE")) {
 		return(rep("BASELINE", length(increment)))
 	} else {
-		if (length(grep("_", year[1]) > 0)) {
-			tmp1 <- sapply(seq_along(increment), function(i) as.numeric(sapply(strsplit(as.character(year), "_"), '[', 2)) + increment[i] - lag)
-			paste(floor(tmp1)-1, tmp1, sep="_")
+		if (grepl("_", year[1])) {
+			tmp1 <- sapply(seq_along(increment), function(i) as.numeric(sapply(strsplit(as.character(year), "_"), '[', 2L)) + increment[i] - lag)
+			paste(floor(tmp1)-1L, tmp1, sep="_")
 		} else {
 			as.character(sapply(seq_along(increment), function(i) as.numeric(year) + increment[i] - lag))
 		}
